/******************************************************************************
 * hWeb --- Simple Web Reader
 ******************************************************************************
 * msg.c : output message
 *
 * Copyright (C) 2007-2016
 *
 * Evgeny Vetrov
 *
 * Fido:     2:6037/7@fidonet
 * Internet: <vea713@gmail.com>, <vea@smekal.ru>
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * Software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *****************************************************************************
 */
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <fidoconf/fidoconf.h>
#include <smapi/msgapi.h>

int main(int argc, char *argv[])
{
	const char *header = "Content-Type:text/plain;charset=cp866\n";
	char *req = NULL;
	char *query = NULL;
	char *areaname = NULL;
	char *msgid = NULL;

	ps_fidoconfig config = NULL;
	ps_area aptr = NULL;
	MSGA *smapi_area = NULL;
	MSGH *smapi_msg = NULL;
	XMSG xmsg;
	char buf[100];
	char *tmp = NULL;

	unsigned int id = 0, i = 0, j = 0, bufLen = 0;

	int ctrlLen;
	byte *ctrlBuff;
	int textLen;
	byte *textBuff;

	char orig[32];
	char dest[32];

	if ( (config = readConfig(getConfigFileName())) == NULL )
	{
		printf("%sStatus: 500\n\nERROR: can`t read config", header);
		return 0;
	}

	if ( (req = getenv("REQUEST_METHOD")) == NULL)
	{
		printf("%sStatus: 500\n\nERROR: can't read env(REQUEST_METHOD)", header);
		disposeConfig(config);
		return 0;
	}

	if ( !strncmp(req, "GET", 3) )
	{
		if ( (query = getenv("QUERY_STRING")) == NULL )
		{
			printf("%sStatus: 500\n\nERROR: can't read env(QUERY_STRING)", header);
			disposeConfig(config);
			return 0;
		}
		for ( i = 0; i < 200 && query[i] != 0; i++ )
		{
			if ( !strncasecmp(&query[i], "AREA=", 5) )
			{
				for (i += 5, areaname = query + i; i < 200 && query[i] != 0 && query[i] != '&'; i++);
				query[i] = 0;
			}
			if ( !strncasecmp(&query[i], "ID=", 3) )
			{
				for (i += 3, msgid = query + i; i < 200 && query[i] != 0 && query[i] != '&'; i++);
				query[i] = 0;
				id = atoi(msgid);
			}
		}
	} else {
		printf("%sStatus: 500\n\nERROR: reque GET metod", header);
		disposeConfig(config);
		return 0;
	}

	if ( areaname != NULL )
	{
		if ( (aptr = getArea(config, areaname)) == NULL )
		{
			printf("%sStatus: 500\n\nERROR: can`t find %s", header, areaname);
			disposeConfig(config);
			return 0;
		}
	} else {
		printf("%sFormat: msg.cgi?area=<areaname>&id=<msgnum>", header);
		disposeConfig(config);
		return 0;
	}

	if ( (smapi_area = MsgOpenArea((byte *) aptr->fileName, MSGAREA_CRIFNEC, aptr->msgbType)) == NULL )
	{
		printf("%sStatus: 500\n\nERROR: can`t open area %s", header, aptr->fileName);
		disposeConfig(config);
		return 0;
	}

	if ( (smapi_msg = MsgOpenMsg(smapi_area, MOPEN_READ, id)) == NULL )
	{
		printf("%sStatus: 500\n\nERROR: can`t open msg #%d", header, id);
		MsgCloseArea(smapi_area);
		disposeConfig(config);
		return 0;
	}

	ctrlLen = MsgGetCtrlLen(smapi_msg);

	if ( (ctrlBuff = malloc(ctrlLen + 1)) == NULL )
	{
		printf("%sStatus: 500\n\nERROR: can`t allocate memory", header);
		MsgCloseMsg(smapi_msg);
		MsgCloseArea(smapi_area);
		disposeConfig(config);
		return 0;
	}

	textLen = MsgGetTextLen(smapi_msg);

	if ( (textBuff = malloc(textLen + 1)) == NULL )
	{
		printf("%sStatus: 500\n\nERROR: can`t allocate memory", header);
		free(ctrlBuff);
		MsgCloseMsg(smapi_msg);
		MsgCloseArea(smapi_area);
		disposeConfig(config);
		return 0;
	}

	if ( MsgReadMsg(smapi_msg, &xmsg, 0L, textLen, textBuff, ctrlLen, ctrlBuff) == -1 )
	{
		printf("%sStatus: 500\n\nERROR: can`t read msg #%d", header, id);
		free(textBuff);
		free(ctrlBuff);
		MsgCloseMsg(smapi_msg);
		MsgCloseArea(smapi_area);
		disposeConfig(config);
		return 0;
	}

	sprintf(orig, "%d:%d/%d.%d", xmsg.orig.zone, xmsg.orig.net, xmsg.orig.node, xmsg.orig.point);
	sprintf(dest, "%d:%d/%d.%d", xmsg.dest.zone, xmsg.dest.net, xmsg.dest.node, xmsg.dest.point);
	for (i = 0; i < ctrlLen; i++)
	{
		if ( ctrlBuff[i] == 1 && !memcmp(&ctrlBuff[i + 1], "MSGID: ", 7) )
		{
			for (j = i + 8; ctrlBuff[j] != 1 && ctrlBuff[j] != '\n' && j < ctrlLen; j++)
				buf[j - i - 8] = ctrlBuff[j];
			if ( id == MsgNumMsg(smapi_area) )
			{
				buf[(j++) - i - 8] = ' ';
				buf[(j++) - i - 8] = 'L';
			}
			buf[j - i - 8] = 0;
			break;
		}
	}

	tmp = getenv("HTTP_IF_NONE_MATCH");

	if ( tmp != NULL && !strcmp(buf, tmp) )
	{
		printf("%sStatus: 304\n\n", header);
		free(textBuff);
		free(ctrlBuff);
		MsgCloseMsg(smapi_msg);
		MsgCloseArea(smapi_area);
		disposeConfig(config);
		return 0;
	}

	printf(
		"%sAge: 3600\nETag: %s\n\n"
		"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n"
		"<html>\n"
		"<head>\n"
		"\t<title>LIST_MESSAGES(%s)</title>\n"
		"\t<LINK href=\"/main.css\" rel=\"stylesheet\" type=\"text/css\" />"
		"</head>\n"
		"<body>\n"
		, header
		, buf
		, areaname
	);

	if (id > 1)
		printf("<div id=\"prev_msg\"><a href=\"msg.cgi?area=%s&amp;msgid=%d\">prev</a></div>\n", aptr->areaName, id - 1);

	if (id < MsgNumMsg(smapi_area))
		printf("<div id=\"next_msg\"><a href=\"msg.cgi?area=%s&amp;msgid=%d\">next</a></div>\n", aptr->areaName, id + 1);

	printf(
		"<table id=\"MSG\">\n"
		"<caption>%s</caption>"
		"<tr><th id=\"from\">From</th><td>%s</td><td>%s</td><th>Date Write</th><td>%d-%02d-%02d %02d:%02d:%02d</td></tr>\n"
		"<tr><th id=\"to\">To</th><td>%s</td><td>%s</td><th>Date Arrived</th><td>%d-%02d-%02d %02d:%02d:%02d</td></tr>\n"
		"<tr><th id=\"subj\">Subj</th><td colspan=4>%s</td></tr>\n"
		"<tr><th id=\"attr\">Attr</th><td colspan=4></td></tr>\n"
		, areaname
		, xmsg.from, orig
		, xmsg.date_written.date.yr + 1980, xmsg.date_written.date.mo, xmsg.date_written.date.da
		, xmsg.date_written.time.hh, xmsg.date_written.time.mm, xmsg.date_written.time.ss
		, xmsg.to, dest
		, xmsg.date_arrived.date.yr + 1980, xmsg.date_arrived.date.mo, xmsg.date_arrived.date.da
		, xmsg.date_arrived.time.hh, xmsg.date_arrived.time.mm, xmsg.date_arrived.time.ss, xmsg.subj
	);

	printf("<tr><td colspan=5 id=\"text\">\n");
	for (i = 0; i < textLen; i += j)
	{
		bufLen = (i + 80 < textLen) ? 80 : textLen - i;
		memcpy(buf, &textBuff[i], bufLen);

		for (j = 0; j < bufLen && buf[j] != 13; j++);

		if ( j == bufLen)
		{
			for (; j > 0 && buf[j] != ' ' && buf[j] != '\000'; j--);
		}

		buf[j++] = 0;
		if ( buf[0] == 0x01 )
		{
			continue;
		}
		if ( !strncasecmp(buf, "SEEN-BY:", 8) )
		{
			continue;
		}
		printf("%s<br/>", buf);
	}
	printf(
		"</td></tr>\n"
		"</table>\n"
		"</body>\n"
		"</html>\n"
	);

	free(textBuff);
	free(ctrlBuff);
	MsgCloseMsg(smapi_msg);
	MsgCloseArea(smapi_area);

	disposeConfig(config);
	return 0;
}
