/******************************************************************************
 * hWeb --- Simple Web Reader
 ******************************************************************************
 * msgs.c : output list messages in area
 *
 * Copyright (C) 2007-2016
 *
 * Evgeny Vetrov
 *
 * Fido:     2:6037/7@fidonet
 * Internet: <vea713@gmail.com>, <vea@smekal.ru>
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * Software is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *****************************************************************************
 */
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <fidoconf/fidoconf.h>
#include <smapi/msgapi.h>

int main(int argc, char *argv[])
{
	const char *header = "Content-Type:text/plain;charset=cp866\n";
	char *req = NULL;
	char *query = NULL;
	char *areaname = NULL;
	char *full = NULL;

	ps_fidoconfig config = NULL;
	ps_area aptr = NULL;
	MSGA *smapi_area = NULL;
	MSGH *smapi_msg = NULL;
	XMSG xmsg;
	byte buf[100];
	char *tmp = NULL;

	unsigned int i = 0;
	int j = 0;

	if ( (config = readConfig(getConfigFileName())) == NULL )
	{
		printf("%sStatus: 500\n\nERROR: can`t read config", header);
		return 0;
	}

	if ( (req = getenv("REQUEST_METHOD")) == NULL )
	{
		printf("%sStatus: 500\n\nERROR: can`t read env(REQUEST_METHOD)", header);
		disposeConfig(config);
		return 0;
	}

	if ( !strncmp(req, "GET", 3) )
	{
		if ( (query = getenv("QUERY_STRING")) == NULL )
		{
			printf("%sStatus: 500\n\nERROR: can`t read env(QUERY_STRING)", header);
			disposeConfig(config);
			return 0;
		}
		for (i = 0; i < 200 && query[i] != 0; i++)
		{
			if ( !strncasecmp(&query[i], "AREA=", 5) )
			{
				for (i += 5, areaname = query + i; i < 200 && query[i] != 0 && query[i] != '&'; i++);
				query[i] = 0;
			}
			if ( !strncasecmp(&query[i], "FULL=", 5) )
			{
				for (i += 5, full = query + i; i < 200 && query[i] != 0 && query[i] != '&'; i++);
				query[i] = 0;
			}
		}
	} else {
		if ( (query = getenv("QUERY_STRING")) == NULL )
		{
			printf("%sStatus: 500\n\nERROR: reque GET metod", header);
			disposeConfig(config);
			return 0;
		}
	}

	if ( areaname != NULL )
	{
		if ( (aptr = getArea(config, areaname)) == NULL )
		{
			printf("%sStatus: 500\n\nERROR: can`t find %s", header, areaname);
			disposeConfig(config);
			return 0;
		}
	} else {
		printf("%sFormat: msgs.cgi?area=<areaname>", header);
		disposeConfig(config);
		return 0;
	}

	if ( (smapi_area = MsgOpenArea((byte *)aptr->fileName, MSGAREA_CRIFNEC, aptr->msgbType)) == NULL )
	{
		printf("%sStatus: 500\n\nERROR: can`t open area %s", header, aptr->fileName);
		disposeConfig(config);
		return 0;
	}

	tmp = getenv("HTTP_IF_NONE_MATCH");

	if ( tmp != NULL && atoi(tmp) == MsgNumMsg(smapi_area) )
	{
		printf("%sStatus: 304\n\n",header);
		MsgCloseArea(smapi_area);
		disposeConfig(config);
		return 0;
	}

	printf(
		"%sAge: 3600\nETag: %lu\n\n"
		"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n"
		"<html>\n"
		"<head>\n"
		"\t<title>LIST_MESSAGES(%s)</title>\n"
		"\t<LINK href=\"/main.css\" rel=\"stylesheet\" type=\"text/css\" />\n"
		"</head>\n"
		"<body>\n"
		"<table id=\"MSGS\">\n"
		"<caption>%s "
		, header
		, MsgNumMsg(smapi_area)
		, areaname
		, areaname
	);

	if (full == NULL)
		printf("<a href=\"?AREA=%s&amp;FULL=1\">FULL</A></caption>\n", areaname);
	else
		printf("<a href=\"?AREA=%s\">SHORT</A></caption>\n", areaname);

	printf("<tr> <th>Num</th> <th>From</th> <th>To</th> <th>Subject</th> <th>Date</th> </tr>\n");

	if (full != NULL)
	{
		j = 0;
	} else if (tmp != NULL) {
		j = atoi(tmp) - 13;
	} else {
		j = MsgNumMsg(smapi_area) - 13;
	}
	if (j < 0)
		j = 0;

	for (i = MsgNumMsg(smapi_area); i > j; i--)
	{
		if ( (smapi_msg = MsgOpenMsg(smapi_area, MOPEN_READ, i)) != NULL )
		{
			if (MsgReadMsg(smapi_msg, &xmsg, 0L, 0L, NULL, 1, buf) != -1)
			{
				printf(
					"<tr%s>"
					"<td><a href=\"msg.cgi?area=%s&amp;msgid=%d\" target=\"MSG\">%d</a></td>"
					"<td>%s</td>" "<td>%s</td>" "<td>%s</td>"
					"<td>%d-%02d-%02d %02d:%02d:%02d</td>"
					"</tr>\n"
					, (tmp != NULL && i > atoi(tmp)) ? " class=\"new\"" : ""
					, areaname, i, i
					, xmsg.from, xmsg.to, xmsg.subj
					, xmsg.date_written.date.yr + 1980, xmsg.date_written.date.mo, xmsg.date_written.date.da
					, xmsg.date_written.time.hh, xmsg.date_written.time.mm, xmsg.date_written.time.ss
				);
			}
		MsgCloseMsg(smapi_msg);
		}
	}

	printf(
		"</table>\n"
		"</body>\n"
		"</html>\n"
	);

	MsgCloseArea(smapi_area);
	disposeConfig(config);
	return 0;
}
